@@ -4,14 +4,9 @@ import numpy as onp
 import jax
 import jax.numpy as jnp
 from jax import jit, vmap, random, grad
-from jax.tree_util import tree_map, tree_multimap
+from jax.tree_util import tree_map
 from jax.scipy.special import logsumexp
-try:
-    from jax.example_libraries import optimizers
-except ImportError:
-    # for jax <= 0.2.24
-    from jax.experimental import optimizers
-
+from jax.example_libraries import optimizers
 
 from dibs.inference.dibs import DiBS
 from dibs.kernel import AdditiveFrobeniusSEKernel, JointAdditiveFrobeniusSEKernel
@@ -32,6 +27,8 @@ class MarginalDiBS(DiBS):
 
     Args:
         x (ndarray): observations of shape ``[n_observations, n_vars]``
+        interv_mask (ndarray, optional): binary matrix of shape ``[n_observations, n_vars]`` indicating
+            whether a given variable was intervened upon in a given sample (intervention = 1, no intervention = 0)
         inference_model: Bayes net inference model defining prior :math:`\\log p(G)`
             and marginal likelihood :math:`\\log p(D | G)`` underlying the inferred posterior.
             Object *has to implement two methods*:
@@ -61,6 +58,7 @@ class MarginalDiBS(DiBS):
     def __init__(self, *,
                  x,
                  inference_model,
+                 interv_mask=None,
                  kernel=AdditiveFrobeniusSEKernel,
                  kernel_param=None,
                  optimizer="rmsprop",
@@ -81,11 +79,16 @@ class MarginalDiBS(DiBS):
         if optimizer_param is None:
             optimizer_param = {"stepsize": 0.005}
 
+        # handle interv mask in observational case
+        if interv_mask is None:
+            interv_mask = jnp.zeros_like(x, dtype=jnp.int32)
+
         # init DiBS superclass methods
         super(MarginalDiBS, self).__init__(
             x=x,
+            interv_mask=interv_mask,
             log_graph_prior=inference_model.log_graph_prior,
-            log_joint_prob=inference_model.observational_log_marginal_prob,
+            log_joint_prob=inference_model.interventional_log_marginal_prob,
             alpha_linear=alpha_linear,
             beta_linear=beta_linear,
             tau=tau,
@@ -98,8 +101,12 @@ class MarginalDiBS(DiBS):
         )
 
         self.inference_model = inference_model
-        self.eltwise_log_marginal_likelihood = vmap(lambda g, x_ho:
-            inference_model.observational_log_marginal_prob(g, None, x_ho, None), (0, None), 0)
+
+        # functions for post-hoc likelihood evaluations
+        self.eltwise_log_marginal_likelihood_observ = vmap(lambda g, x_ho:
+             inference_model.interventional_log_marginal_prob(g, None, x_ho, jnp.zeros_like(self.x), None), (0, None), 0)
+        self.eltwise_log_marginal_likelihood_interv = vmap(lambda g, x_ho, interv_msk_ho:
+             inference_model.interventional_log_marginal_prob(g, None, x_ho, interv_msk_ho, None), (0, None, None), 0)
 
         self.kernel = kernel(**kernel_param)
 
@@ -298,7 +305,7 @@ class MarginalDiBS(DiBS):
         """Execute particle update steps for all particles in parallel using `vmap` functions"""
         # faster if for-loop is functionally pure and compiled, so only interrupt for callback
         callback_every = callback_every or steps
-        for t in range(0, steps, callback_every):
+        for t in (range(0, steps, callback_every) if steps else range(0)):
 
             # perform sequence of SVGD steps
             opt_state_z, key, sf_baseline = self._svgd_loop(t, callback_every, (opt_state_z, key, sf_baseline))
@@ -356,7 +363,8 @@ class MarginalDiBS(DiBS):
         N, _, _ = g.shape
 
         # mixture weighted by respective marginal probabilities
-        eltwise_log_marginal_target = vmap(lambda single_g: self.log_joint_prob(single_g, None, self.x, None), 0, 0)
+        eltwise_log_marginal_target = vmap(lambda single_g:
+                                           self.log_joint_prob(single_g, None, self.x, self.interv_mask, None), 0, 0)
         logp = eltwise_log_marginal_target(g)
         logp -= logsumexp(logp)
 
@@ -378,6 +386,8 @@ class JointDiBS(DiBS):
 
     Args:
         x (ndarray): observations of shape ``[n_observations, n_vars]``
+        interv_mask (ndarray, optional): binary matrix of shape ``[n_observations, n_vars]`` indicating
+            whether a given variable was intervened upon in a given sample (intervention = 1, no intervention = 0)
         inference_model: Bayes net inference model defining prior :math:`\\log p(G)`
             and joint likelihood :math:`\\log p(\\Theta, D | G) = \\log p(\\Theta | G) + \\log p(D | G, \\Theta``
             underlying the inferred posterior. Object *has to implement two methods*:
@@ -408,6 +418,7 @@ class JointDiBS(DiBS):
     def __init__(self, *,
                  x,
                  inference_model,
+                 interv_mask=None,
                  kernel=JointAdditiveFrobeniusSEKernel,
                  kernel_param=None,
                  optimizer="rmsprop",
@@ -428,11 +439,16 @@ class JointDiBS(DiBS):
         if optimizer_param is None:
             optimizer_param = {"stepsize": 0.005}
 
+        # handle interv mask in observational case
+        if interv_mask is None:
+            interv_mask = jnp.zeros_like(x, dtype=jnp.int32)
+
         # init DiBS superclass methods
         super(JointDiBS, self).__init__(
             x=x,
+            interv_mask=interv_mask,
             log_graph_prior=inference_model.log_graph_prior,
-            log_joint_prob=inference_model.observational_log_joint_prob,
+            log_joint_prob=inference_model.interventional_log_joint_prob,
             alpha_linear=alpha_linear,
             beta_linear=beta_linear,
             tau=tau,
@@ -445,8 +461,12 @@ class JointDiBS(DiBS):
         )
 
         self.inference_model = inference_model
-        self.eltwise_log_likelihood = vmap(lambda g, theta, x_ho:
-            inference_model.observational_log_joint_prob(g, theta, x_ho, None), (0, 0, None), 0)
+
+        # functions for post-hoc likelihood evaluations
+        self.eltwise_log_likelihood_observ = vmap(lambda g, theta, x_ho:
+            inference_model.interventional_log_joint_prob(g, theta, x_ho, jnp.zeros_like(self.x), None), (0, 0, None), 0)
+        self.eltwise_log_likelihood_interv = vmap(lambda g, theta, x_ho, interv_msk_ho:
+            inference_model.interventional_log_joint_prob(g, theta, x_ho, interv_msk_ho, None), (0, 0, None, None), 0)
 
         self.kernel = kernel(**kernel_param)
 
@@ -456,7 +476,6 @@ class JointDiBS(DiBS):
             self.opt = optimizers.rmsprop(optimizer_param['stepsize'])
         else:
             raise ValueError()
-       
 
     def _sample_initial_random_particles(self, *, key, n_particles, n_dim=None):
         """
@@ -473,9 +492,9 @@ class JointDiBS(DiBS):
         # default full rank
         if n_dim is None:
             n_dim = self.n_vars
-        
-        # std like Gaussian prior over Z           
-        std = self.latent_prior_std  or (1.0 / jnp.sqrt(n_dim))
+
+        # std like Gaussian prior over Z
+        std = self.latent_prior_std or (1.0 / jnp.sqrt(n_dim))
 
         # sample from parameter prior
         key, subk = random.split(key)
@@ -504,7 +523,7 @@ class JointDiBS(DiBS):
         return self.kernel.eval(
             x_latent=x_latent, x_theta=x_theta,
             y_latent=y_latent, y_theta=y_theta)
-    
+
 
     def _f_kernel_mat(self, x_latents, x_thetas, y_latents, y_thetas):
         """
@@ -522,7 +541,7 @@ class JointDiBS(DiBS):
         return vmap(vmap(self._f_kernel, (None, None, 0, 0), 0),
                     (0, 0, None, None), 0)(x_latents, x_thetas, y_latents, y_thetas)
 
-    
+
     def _eltwise_grad_kernel_z(self, x_latents, x_thetas, y_latent, y_theta):
         """
         Computes gradient :math:`\\nabla_Z k((Z, \\Theta), (Z', \\Theta'))` elementwise
@@ -536,8 +555,8 @@ class JointDiBS(DiBS):
 
         Returns:
             batch of gradients of shape ``[n_particles, d, k, 2]``
-        
-        """        
+
+        """
         grad_kernel_z = grad(self._f_kernel, 0)
         return vmap(grad_kernel_z, (0, 0, None, None), 0)(x_latents, x_thetas, y_latent, y_theta)
 
@@ -578,7 +597,7 @@ class JointDiBS(DiBS):
         Returns
             transform vector of shape ``[d, k, 2]`` for the particle ``single_z``
         """
-    
+
         # compute terms in sum
         weighted_gradient_ascent = kxx[..., None, None, None] * grad_log_prob_z
         repulsion = self._eltwise_grad_kernel_z(z, theta, single_z, single_theta)
@@ -618,21 +637,18 @@ class JointDiBS(DiBS):
         Returns:
             transform vector PyTree with leading dim ``n_particles`` for the particle ``single_theta``
         """
-    
+
         # compute terms in sum
         weighted_gradient_ascent = tree_map(
-            lambda leaf_theta_grad: 
-                expand_by(kxx, leaf_theta_grad.ndim - 1) * leaf_theta_grad, 
+            lambda leaf_theta_grad: expand_by(kxx, leaf_theta_grad.ndim - 1) * leaf_theta_grad,
             grad_log_prob_theta)
-        
+
         repulsion = self._eltwise_grad_kernel_theta(z, theta, single_z, single_theta)
 
         # average and negate (for optimizer)
-        return  tree_multimap(
-            lambda grad_asc_leaf, repuls_leaf: 
-                - (grad_asc_leaf + repuls_leaf).mean(axis=0), 
-            weighted_gradient_ascent, 
-            repulsion)
+        return  tree_map(
+            lambda grad_asc_leaf, repuls_leaf: - (grad_asc_leaf + repuls_leaf).mean(axis=0),
+            weighted_gradient_ascent, repulsion)
 
 
     def _parallel_update_theta(self, *args):
@@ -662,8 +678,8 @@ class JointDiBS(DiBS):
             the updated inputs ``opt_state_z``, ``opt_state_theta``, ``key``, ``sf_baseline``
         """
 
-        z = self.get_params(opt_state_z) # [n_particles, d, k, 2]
-        theta = self.get_params(opt_state_theta) # PyTree with `n_particles` leading dim
+        z = self.get_params(opt_state_z)  # [n_particles, d, k, 2]
+        theta = self.get_params(opt_state_theta)  # PyTree with `n_particles` leading dim
         n_particles = z.shape[0]
 
         # d/dtheta log p(theta, D | z)
@@ -678,9 +694,9 @@ class JointDiBS(DiBS):
         key, *batch_subk = random.split(key, n_particles + 1)
         dz_log_prior = self.eltwise_grad_latent_prior(z, jnp.array(batch_subk), t)
 
-        # d/dz log p(z, theta, D) = d/dz log p(z)  + log p(theta, D | z) 
+        # d/dz log p(z, theta, D) = d/dz log p(z)  + log p(theta, D | z)
         dz_log_prob = dz_log_prior + dz_log_likelihood
-        
+
         # k((z, theta), (z, theta)) for all particles
         kxx = self._f_kernel_mat(z, theta, z, theta)
 
@@ -694,8 +710,8 @@ class JointDiBS(DiBS):
         opt_state_theta = self.opt_update(t, phi_theta, opt_state_theta)
 
         return opt_state_z, opt_state_theta, key, sf_baseline
-    
-    
+
+
     # this is the crucial @jit
     @functools.partial(jit, static_argnums=(0, 2))
     def _svgd_loop(self, start, n_steps, init):
@@ -743,11 +759,12 @@ class JointDiBS(DiBS):
         """Execute particle update steps for all particles in parallel using `vmap` functions"""
         # faster if for-loop is functionally pure and compiled, so only interrupt for callback
         callback_every = callback_every or steps
-        for t in range(0, steps, callback_every):
+        for t in (range(0, steps, callback_every) if steps else range(0)):
 
             # perform sequence of SVGD steps
             opt_state_z, opt_state_theta, key, sf_baseline = self._svgd_loop(t, callback_every,
-                                                                             (opt_state_z, opt_state_theta, key, sf_baseline))
+                                                                             (opt_state_z, opt_state_theta, key,
+                                                                              sf_baseline))
 
             # callback
             if callback:
@@ -810,7 +827,8 @@ class JointDiBS(DiBS):
 
         # mixture weighted by respective joint probabilities
         eltwise_log_joint_target = vmap(lambda single_g, single_theta:
-                                        self.log_joint_prob(single_g, single_theta, self.x, None), (0, 0), 0)
+                                        self.log_joint_prob(single_g, single_theta, self.x, self.interv_mask, None),
+                                        (0, 0), 0)
         logp = eltwise_log_joint_target(g, theta)
         logp -= logsumexp(logp)
 
